// Algoritmul lui Kruskal
// O(E log E)
// Det arborelui partial de cost minim intr-un graf
// Minimum spanning tree (arborele minim de acoperire)

#include <fstream>
#include <algorithm>
#include <vector>
using namespace std;
ofstream fout("graf.out");


struct Edge {
	int x, y, w;
//	Edge(int a = 0, int b = 0, int c = 0)
//        : x(a), y(b), w(c)
//    {}
    bool operator < (const Edge& e) const
    {
        return w < e.w;
    }
};

vector<Edge> edges;     // memoreaza graful

int cost_min;      // costul APM - MST (arb part de cost min)
vector<Edge> arb;   // APM
int n;

/// Paduri de multimi disjuncte
vector<int> t;
void Union(int x, int y);
void Init();
int Find(int x);

void ReadGraph();
void Kruskal();
void Print();

int main()
{
	ReadGraph();
	Init();
	Kruskal();
	Print();

	return 0;
}

void ReadGraph()
{
	ifstream fin("kruskal.in");
	fin >> n;
	int v1, v2, cost;
	while ( fin >> v1 >> v2 >> cost )
        edges.emplace_back(v1, v2, cost);
	fin.close();
}

void Kruskal()
{
	sort(edges.begin(), edges.end());
	int k = 0;  // nr muchii in APM
	int c1, c2; // componetele conexe ale varfurilor muchiei

	for (const auto& edge : edges) /// O(m)
	{
        c1 = Find(edge.x); c2 = Find(edge.y);
		if ( c1 != c2)  // daca nu formeaza ciclu
		{
			cost_min += edge.w;
			arb.push_back(edge);   // adaug muchia e in APM
			Union(c1, c2);

			if (k == n - 1)  break;
		}
    }
}

void Print()
{
	fout << cost_min << '\n';
	for (const auto& edge : arb)
		fout << edge.x << " " << e.y << " " << e.w << endl;
	fout.close();
}

void Init()
{
	t = vector<int>(n + 1);
	for (int i = 1; i <= n; ++i)
        t[i] = i;
}

void Union(int x, int y)
{
    t[x] = y;
}

int Find(int x)
{
    if (x == t[x]) return x;
    return t[x] = Find(t[x]);
}



