/* Algoritmul lui Kruskal
   O(m * log m)
   Det arborelui partial de cost minim intr-un graf
   Minimum spanning tree (arborele minim de acoperire)
*/

#include <fstream>
#include <algorithm>
#include <vector>
#include <tuple>
using namespace std;

ifstream fin("kruskal.in");
ofstream fout("kruskal.out");

               // < w,   x,   y>
using Edge = tuple<int, int, int>;
vector<Edge> edges;     // memoreaza graful
vector<int> t;
int cost_min;           // costul APM - MST (arb part de cost min)
vector<Edge> arb;       // APM
int n;

void Union(int x, int y);
void Init();
int Find(int x);
void ReadGraph();
void Kruskal();
void Print();

int main()
{
	ReadGraph();
	Init();
	Kruskal();
	Print();

	return 0;
}

void ReadGraph()
{
	fin >> n;
	int x, y, w;
	while (fin >> x >> y >> w)
        edges.emplace_back(w, x, y);
	fin.close();
}

void Kruskal()
{
	sort(edges.begin(), edges.end());
	int k = 0;  // nr muchii in APM
	int c1, c2; // componetele conexe ale varfurilor muchiei

	for (auto [w, x, y] : edges)
	{
        c1 = Find(x); c2 = Find(y);
		if ( c1 != c2)  // daca nu formeaza ciclu
		{
			cost_min += w;
			arb.emplace_back(x, y, w);   // adaug muchia e in APM
			Union(c1, c2);

			if (k == n - 1)  break;
		}
    }
}

void Print()
{
	fout << cost_min << '\n';
	for (auto [x, y, w] : arb)
		fout << x << " " << y << " " << w << '\n';
	fout.close();
}

void Init()
{
	t = vector<int>(n + 1);
	for (int i = 1; i <= n; ++i)
        t[i] = i;
}

void Union(int x, int y)
{
    t[x] = y;
}

int Find(int x)
{
    if (x == t[x]) return x;
    return t[x] = Find(t[x]);
}



