// Timp: O(Lung) / query              Lung - lungimea unui cuvant
// Memorie: O(LungTot * |Sigma|)      LungTot = lungimea totala a cuvintelor din lista
#include <iostream>
#include <fstream>
#include <cstring>
using namespace std;

const int Sigma = 26;

struct Nod {
    Nod()
    {
        nr_cuv = nr_sons = 0;
        memset (son, 0, sizeof(son));
    }

    int nr_cuv, nr_sons;       // nr_cuv - nr de cuvinte care se termina pe acest nod
    Nod* son[Sigma];
};

Nod *T = new Nod;
char sir[Sigma];
int tip;

void Insert(Nod *nod, char *s)
{
    if (*s == '\0')
    {
        nod->nr_cuv++;   // pe nodul curent se mai termina un cuvant (ultima litera e muchia spre tata)
        return;
    }

    if (nod->son[*s - 'a'] == 0)         // daca nu exista fiul coresponzator caracterului
    {
        nod->son[*s - 'a'] = new Nod;  // se creaza nodul fiu
        nod->nr_sons++;
    }

    Insert(nod->son[*s - 'a'], s + 1);
}

bool Del(Nod *nod, char *s)   // returneaza true daca nodul curent se sterge din arbore (se dezaloca cu delete)
{
    if (*s == '\0')            // scae nr de cuvinte care se termina pe nodul curent
        nod->nr_cuv--;
    else
        if (Del(nod->son[*s - 'a'], s + 1))  // daca fiul corespunzator literei a fost sters din arbore
        {
            nod->son[*s - 'a'] = 0;         // plasam null in pointerul corespunzator
            nod->nr_sons--;                 // si decrementam numarul de fii
        }

    if (nod->nr_cuv == 0 && nod->nr_sons == 0 && nod != T) // daca nod devine frunza si nu se termina niciun cuvant pe el
    {
        delete nod;                                          // se sterge
        return true;
    }

    return false;
}

int Count(Nod *nod, char *s)
{
    if (*s == '\0')
        return nod->nr_cuv;

    if (nod->son[*s - 'a'])  // daca exista litera corespunzatoare, continuam
            return Count(nod->son[*s - 'a'], s + 1);
    return 0;                // daca nu, nu exista cuvantul
}

int Prefix(Nod *nod, char *s, int k)
{
    if ( *s == '\0' || nod->son[*s - 'a'] == 0 )
        return k;

    return Prefix(nod->son[*s - 'a'], s + 1, k + 1);
}

int main()
{
    ifstream fin("trie.in");
	ofstream fout("trie.out");

    while (fin.getline(sir, Sigma))
    {
        switch (sir[0])
        {
            case '0': Insert(T, sir + 2);                     break;
            case '1': Del(T, sir + 2);                        break;
            case '2': fout << Count(T, sir + 2) << "\n";      break;
            case '3': fout << Prefix(T, sir + 2, 0) << "\n";  break;

            default: cout << "Eroare!\n";   break;
        }
    }

    return 0;
}
